#! /bin/bash

###### Customize

# Location of service and service log directories
VARSERVICEROOT="/var/service"

# PATH containing basic system utilities
# such as grep, egrep, sed, awk.
PATH=/command:/command:/bin:/usr/bin:/usr/sbin

# Location of the rlc-* scripts of the runlevelconf package
RUNLEVELCONF_BIN="/command"

# location of dt-service.init and dt-list
DTRUNCONF="/var/service/dt-run"

# How to find the current runlevel
# On Solaris, put ` $(who -r|awk '{ print $3 }') '
RUNLEVEL="$(/sbin/runlevel | cut -d' ' -f2)"

##### End Customize

export PATH=/command:$RUNLEVELCONF_BIN:$PATH

for i in $VARSERVICEROOT /service $DTRUNCONF; do
    if [ ! -d "$i" ]; then
	echo "$(basename $0): $i does not exist"
	exit 1
    fi
done

if [ ! -f $DTRUNCONF/dt-list ]; then
    echo "$(basename $0):  $DTRUNCONF/dt-list does not exist"
    exit 1
fi

DTLIST=$DTRUNCONF/dt-list

if [ ! -x "$RUNLEVELCONF_BIN/rlc-add" ]; then
    echo "$(basename $0): I would not be able to execute rlc-add"
    exit 1
elif [ ! -x "$RUNLEVELCONF_BIN/rlc-doit" ]; then
    echo "$(basename $0): I would not be able to execute rlc-doit"
    exit 1
fi

addit() {
    # usage: addit service runlevels parents
    service="$1"
    runlevels="$2"
    parents="$3"

    if ! grep -q "^$service:" $DTLIST; then
	echo "$(basename $0): $service is not in dt-list"
	return 1
    fi

    if [ ! -d "$VARSERVICEROOT/$service" ] ; then
	echo "$(basename $0): $VARSERVICEROOT/$service does not exist"
	return 1
    fi


    if [ -d "/service/$service" ] && svok "/service/$service" ; then
	echo "$(basename $0): $service is already added"
	echo "If you do not agree, try using remove-service.sh first"
	return 1
    fi

    
    if [ -f "$DTRUNCONF/dt-service.init" ]; then
	echo "Creating /command/$service.init ..."
	ln -sf $DTRUNCONF/dt-service.init /command/$service.init
    else
	echo "$(basename $0): $DTRUNCONF/dt-service.init does not exist"
	return 1
    fi
    
    echo "Adding $service to runlevels $runlevels"
    for i in $(echo $runlevels | sed 's},} }g' ); do
	echo "Runlevel $i:"

	for parent in $(echo $parents | sed 's},} }g' ); do
	    
	    if rlc-check ${i}s $parent | grep -q yet; then    
		echo "Parent $parent is not yet added to ${i}s"
                return 1
            fi
	done
	echo "Running rlc-add..."
	rlc-add ${i}s $service $(echo $parents | sed 's},} }g' )

	echo "Running rlc-doit; might take a while..."
	rlc-doit ${i}s
	if ! [ $? = 0 -o $? = 2 ]; then
	    return 1
	fi
    done

    if ! rlc-check ${RUNLEVEL}s $service| grep -q "already"; then
	touch $VARSERVICEROOT/$service/down
	touch $VARSERVICEROOT/$service/log/down
    else
	rm -f $VARSERVICEROOT/$service/log/down
	rm -f $VARSERVICEROOT/$service/down
    fi
    
    echo "Creating the link /service/$service"
    ln -sf $VARSERVICEROOT/$service /service
     
}

add_one() {
    # Usage: add_one service
    service="$1"
    runlevels=$(awk -F: "/^$service:/"'{ print $2 }' $DTLIST)
    parents=$(awk -F: "/^$service:/"'{ print $3 }' $DTLIST)

    if ! addit $service "$runlevels" "$parents"; then
	echo "Problems adding $service; giving up on $service"
	return 1
    fi
}

help() {
    echo "Usage: "
    echo "$(basename $0) service [service...]"
    echo "$(basename $0) help"
    echo "$(basename $0) list"
}

list() {
    awk -F: '!/#/ { print $1 }' $DTLIST
}

case "$#" in
    0)
	help
	;;
    1)
	if [ $1 = help ]; then
	    help
	elif [ $1 = list ]; then
	    list
	else
	    add_one $1
	fi
	;;
    *)	
	for i in $@; do
	    add_one $i
	done
esac


