#! /bin/bash

###### Customize

# svscan's directory; not customizable anymore because
# we use svscanboot 
SERVICEROOT=/service

# Location of service and service log directories
VARSERVICEROOT="/var/service"

# PATH containing basic system utilities
# such as grep, egrep, sed, awk.
PATH=/command:/command:/bin:/usr/bin:/usr/sbin

# Location of the rlc-* scripts of the runlevelconf package
RUNLEVELCONF_BIN="/command"

# location of dt-service.init
DTRUNCONF="/var/service/dt-run"

# How to find the current runlevel
# On Solaris, put ` $(who -r|awk '{ print $3 }') '
RUNLEVEL="$(/sbin/runlevel | cut -d' ' -f2)"

##### End Customize

# svscan's directory; not customizable anymore because
# we use svscanboot 
SERVICEROOT=/service

export PATH=/command:$RUNLEVELCONF_BIN:$PATH

for i in $VARSERVICEROOT $SERVICEROOT; do
    if [ ! -d "$i" ]; then
	echo "$(basename $0): $i does not exist"
	exit 1
    fi
done

if [ ! -f $DTRUNCONF/dt-list ]; then
    echo "$(basename $0):  $DTRUNCONF/dt-list does not exist"
    exit 1
fi

DTLIST=$DTRUNCONF/dt-list

if [ ! -x "$RUNLEVELCONF_BIN/rlc-remove" ]; then
    echo "$(basename $0): I would not be able to execute rlc-remove"
    exit 1
elif [ ! -x "$RUNLEVELCONF_BIN/rlc-doit" ]; then
    echo "$(basename $0): I would not be able to execute rlc-doit"
    exit 1
fi

removeit() {
    # Usage: removeit service runlevels
    service="$1"
    runlevels="$2"
    SERVICEDIR=$SERVICEROOT/$service
    SERVICELOGDIR=$SERVICEDIR/log
    VARSERVICEDIR=$VARSERVICEROOT/$service
    VARSERVICELOGDIR=$VARSERVICEDIR/log

    if ! grep -q "^$service:" $DTLIST; then
	echo "$(basename $0): $service is not in dt-list"
	return 1
    fi

    if [ ! -d $SERVICEDIR ] || ! svok $SERVICEDIR ; then

	    echo "Cannot control $SERVICE in $SERVICEDIR"
	    echo "Now trying in $VARSERVICEDIR:"

	    if [ ! -d $VARSERVICEDIR ] || ! svok $VARSERVICEDIR ; then
		echo "This is not working either; I can now only clean up"
	    else
		if ! $service.init $VARSERVICEROOT stop; then
		    echo "Could not stop $service; needs killing"
		    svc -k $VARSERVICEDIR
		    echo "$service is killed"
		    svc -d $VARSERVICELOGDIR
		fi
	    fi
	    	
	    
    else
	if ! $service.init stop; then
	echo "Could not stop $service; needs killing"
	    svc -k $SERVICEDIR
	    echo "$service is killed"
	    svc -d $SERVICELOGDIR
	fi
	    
   fi

    echo "Deleting command/$service.init ..."
    rm -f /command/$service.init

    echo "Removing the link $SERVICEDIR"
    rm -f $SERVICEDIR

    if [ -d "$VARSERVICEDIR" ] && svok "$VARSERVICEDIR"; then
	echo "Stopping supervises for $service ..."
	svc -x "$VARSERVICEDIR"
    fi

    if [ -d "$VARSERVICELOGDIR" ] && svok "$VARSERVICELOGDIR"; then
	echo "Stopping supervises for $service/log ..."
	svc -x "$VARSERVICELOGDIR"
    fi    

    echo "Removing $service from runlevels $runlevels"
    for i in $(echo $runlevels | sed 's},} }g' ); do
	echo "Runlevel $i:"
	echo "Running rlc-remove..."
	rlc-remove ${i}s $service

	echo "Running rlc-doit; might take a while..."
	rlc-doit ${i}s
    done
    
}

remove_one() {
    # Usage: remove_one service
    service="$1"
    runlevels=$(awk -F: "/^$service:/"'{ print $2 }' $DTLIST)

    if ! removeit $service "$runlevels"; then
	echo "Problems removing $service; giving up on $service"
	return 1	
    fi
}


help() {
    echo "Usage: "
    echo "$(basename $0) service [service...]"
    echo "$(basename $0) help"
    echo "$(basename $0) list"
}

list() {
    awk -F: '!/#/ { print $1 }' $DTLIST
}

case "$#" in
    0)
	help
	;;
    1)
	if [ $1 = help ]; then
	    help
	elif [ $1 = list ]; then
	    list
	else
	    remove_one $1
	fi
	;;
    *)
	for i in $@; do
	    remove_one $i
	done
esac
